/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_predrv_user.c
* Version      : 
* Device(s)    : 
* Tool-Chain   : 
* Description  : This file implements device driver for pre-driver module.
* Creation Date: 2016/07/05
***********************************************************************************************************************/

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_predrv_reg.h"
#include "r_cg_predrv.h"
#include "r_cg_sau.h"
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: R_PreDrvReg_Read
* Description  : 
* Arguments    : uint8_t address  : predriver register address
*                uint8_t *rd_data : predriver register read data
* Return Value : status -
*                    MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS R_PreDrvReg_Read(uint8_t address, uint8_t *rd_data)
{
    MD_STATUS status = MD_OK;

    uint8_t tx_buf[REG_BUFF_SIZE];
    uint8_t rx_buf[REG_BUFF_SIZE];

    /* init */
    address   = address << 1;                                           /* BitShift */
    tx_buf[0] = address | 0x80;                                         /* MSB bit = 1 (read) */
    tx_buf[1] = 0;
    rx_buf[0] = 0;
    rx_buf[1] = 0;

    /* register read */
    status = R_CSI00_Send_Receive_SPI_mode(tx_buf, REG_BUFF_SIZE, rx_buf, SPI_WAIT_MODE);

    *rd_data = rx_buf[1];

    return (status);
}

/***********************************************************************************************************************
* Function Name: R_PreDrvReg_Write
* Description  : 
* Arguments    : uint8_t address : predriver register address
*                uint8_t wr_data : predriver register write data
* Return Value : status -
*                    MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS R_PreDrvReg_Write(uint8_t address, uint8_t wr_data)
{
    MD_STATUS status = MD_OK;

    uint8_t tx_buf[REG_BUFF_SIZE];
    uint8_t rx_buf[REG_BUFF_SIZE];

    /* init */
    address   = address << 1;                                           /* BitShift */
    tx_buf[0] = address;
    tx_buf[1] = wr_data;
    rx_buf[0] = 0;
    rx_buf[1] = 0;

    /* register write */
    status = R_CSI00_Send_Receive_SPI_mode(tx_buf, REG_BUFF_SIZE, rx_buf, SPI_WAIT_MODE);

    return (status);
}

