/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2017-2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_tmrg.h
* Version      : Code Generator for RL78/G1F V1.00.01.03 [07 Mar 2016]
* Device(s)    : R5F11BLE
* Tool-Chain   : CA78K0R
* Description  : This file implements device driver for TMRG module.
* Creation Date: 2017/06/01
***********************************************************************************************************************/
#ifndef TMRG_H
#define TMRG_H

/***********************************************************************************************************************
Macro definitions (Register bit)
***********************************************************************************************************************/
/*
    Timer RG status resister (TRGSR) 
*/
/* Count direction flag (TRGDIRF) */
#define _00_TMRG_REGISTER_DECREMENTED               (0x00U)             /* TRG register is decremented */
#define _10_TMRG_REGISTER_INCREMENTED               (0x10U)             /* TRG register is incremented */
/* Overflow flag (TRGOVF) */
#define _00_TMRG_WRITE0_OVERFLOW                    (0x00U)             /* write 0 after reading */
#define _08_TMRG_REGISTER_OVERFLOW                  (0x08U)             /* when the TRG register overflows */
/* Underflow flag (TRGUDF) */
#define _00_TMRG_WRITE0_UNDERFLOW                   (0x00U)             /* write 0 after reading */
#define _04_TMRG_REGISTER_UNDERFLOW                 (0x04U)             /* when the TRG register underflows */
/* Input-capture/compare-match interrupt enable B (TRGIMIEB) */
#define _00_TMRG_WRITE0_MATCHB                      (0x00U)             /* write 0 after reading */
#define _02_TMRG_TRGIOB_MATCH                       (0x02U)             /* input edge of TRGIOB pin/When the values of registers TRG and TRGGRB match */
/* Input-capture/compare-match interrupt enable A (TRGIMIEA) */
#define _00_TMRG_WRITE0_MATCHA                      (0x00U)             /* write 0 after reading */
#define _01_TMRG_TRGIOA_MATCH                       (0x01U)             /* input edge of TRGIOA pin/When the values of registers TRG and TRGGRA match */

/*
    Timer RG mode resister (TRGMR) 
*/
/* TRG count start (TRGSTART) */
#define _00_TMRG_COUNT_STOP                         (0x00U)             /* count stops */
#define _80_TMRG_COUNT_START                        (0x80U)             /* count starts */
/* ELC input capture request select (TRGELCICE) */
#define _00_TMRG_SIGNALB_SELECTED                   (0x00U)             /* external output signal B/digital filtering signal B is selected */
#define _40_TMRG_ELC_SELECTED                       (0x40U)             /* event input (input capture) from ELC is selected */
/* Digital filter function clock select (TRGDFCK1,TRGDFCK0) */
#define _00_TMRG_fCLK_32_SELECTED                   (0x00U)             /* fCLK/32 */
#define _10_TMRG_fCLK_8_SELECTED                    (0x10U)             /* fCLK/8 */
#define _20_TMRG_fCLK_SELECTED                      (0x20U)             /* fCLK */
#define _30_TMRG_REGISTER_SELECT                    (0x30U)             /* clock selected by bits TRGTCK0 to TRGTCK2 in TRGCR register */
/* Digital filer function select for TRGIOB pin (TRGDFB) */
#define _00_TMRG_TRGIOB_FILTER_USED                 (0x00U)             /* digital filter function not used */
#define _08_TMRG_TRGIOB_FILTER_UNUSED               (0x08U)             /* digital filter function used */
/* Digital filer function select for TRGIOA pin (TRGDFA) */
#define _00_TMRG_TRGIOA_FILTER_USED                 (0x00U)             /* digital filter function not used */
#define _04_TMRG_TRGIOA_FILTER_UNUSED               (0x04U)             /* digital filter function used */
/* Phase counting mode select (TRGMDF) */
#define _00_TMRG_INCREMENT_MODE                     (0x00U)             /* increment */
#define _02_TMRG_COUNTING_MODE                      (0x02U)             /* phase counting mode */
/* PWM mode select (TRGPWM) */
#define _00_TMRG_TINER_MODE                         (0x00U)             /* timer Mode */
#define _01_TMRG_PWM_MODE                           (0x01U)             /* pwm mode */

/*
    Timer RG count control resister (TRGCNTC) 
*/
/* Counter enable 7 (CNTEN7) */
#define _00_TMRG_COUNTER7_DISABLE                   (0x00U)             /* disabled */
#define _80_TMRG_COUNTER7_INCREMNET                 (0x80U)             /* increment */
/* Counter enable 6 (CNTEN6) */
#define _00_TMRG_COUNTER6_DISABLE                   (0x00U)             /* disabled */
#define _40_TMRG_COUNTER6_INCREMNET                 (0x40U)             /* increment */
/* Counter enable 5 (CNTEN5) */
#define _00_TMRG_COUNTER5_DISABLE                   (0x00U)             /* disabled */
#define _20_TMRG_COUNTER5_INCREMNET                 (0x20U)             /* increment */
/* Counter enable 4 (CNTEN4) */
#define _00_TMRG_COUNTER4_DISABLE                   (0x00U)             /* disabled */
#define _10_TMRG_COUNTER4_INCREMNET                 (0x10U)             /* increment */
/* Counter enable 3 (CNTEN3) */
#define _00_TMRG_COUNTER3_DISABLE                   (0x00U)             /* disabled */
#define _08_TMRG_COUNTER3_INCREMNET                 (0x08U)             /* increment */
/* Counter enable 2 (CNTEN2) */
#define _00_TMRG_COUNTER2_DISABLE                   (0x00U)             /* disabled */
#define _04_TMRG_COUNTER2_INCREMNET                 (0x04U)             /* increment */
/* Counter enable 1 (CNTEN1) */
#define _00_TMRG_COUNTER1_DISABLE                   (0x00U)             /* disabled */
#define _02_TMRG_COUNTER1_INCREMNET                 (0x02U)             /* increment */
/* Counter enable 0 (CNTEN0) */
#define _00_TMRG_COUNTER0_DISABLE                   (0x00U)             /* disabled */
#define _01_TMRG_COUNTER0_INCREMNET                 (0x01U)             /* increment */

/*
    Timer RG control resister (TRGCR) 
*/
/* TRG register clear source select (TRGCCLR1,TRGCCLR0) */
#define _00_TMRG_CLEAR_DISABLEED                    (0x00U)             /* clear disabled */
#define _20_TMRG_CLEAR_TRGGRA                       (0x20U)             /* clear by input capture or compare match with TRGGRA */
#define _40_TMRG_CLEAR_TRGGRB                       (0x40U)             /* clear by input capture or compare match with TRGGRB */
/* External clock active edge select (TRGCKEG1,TRGCKEG0) */
#define _00_TMRG_COUNT_RISING                       (0x00U)             /* count at the rising edge */
#define _08_TMRG_COUNT_FALLING                      (0x08U)             /* count at the falling edge */
#define _10_TMRG_COUNT_BOTH                         (0x10U)             /* count at both the rising/falling edges */
/* Count source select (TRGTCK2,TRGTCK1,TRGTCK0) */
#define _00_TMRG_fCLK_SELECTED                      (0x00U)             /* fCLK */
#define _01_TMRG_fCLK_2_SELECTED                    (0x01U)             /* fCLK/2 */
#define _02_TMRG_fCLK_4_SELECTED                    (0x02U)             /* fCLK/4 */
#define _03_TMRG_fCLK_8_SELECTED                    (0x03U)             /* fCLK/8 */
#define _04_TMRG_fCLK_32_SELECTED                   (0x04U)             /* fCLK/32 */
#define _05_TMRG_TRGCLKA_INPUT                      (0x05U)             /* TRGCLKA input */
#define _07_TMRG_TRGCLKB_INPUT                      (0x07U)             /* TRGCLKB input */

/*
    Timer RG interrupt enable resister (TRGIER) 
*/
/* Overflow interrupt enable (TRGOVIE) */
#define _00_TMRG_INTERRUPT_TRGOVF_DISABLE           (0x00U)             /* interrupt by TRGOVF bit disabled */
#define _08_TMRG_INTERRUPT_TRGOVF_ENABLE            (0x08U)             /* interrupt by TRGOVF bit enabled */
/* Underflow interrupt enable (TRGUDIE) */
#define _00_TMRG_INTERRUPT_TRGUDF_DISABLE           (0x00U)             /* interrupt by TRGUDF bit disabled */
#define _04_TMRG_INTERRUPT_TRGUDF_ENABLE            (0x04U)             /* interrupt by TRGUDF bit enabled */
/* Input-capture/compare-match interrupt enable B (TRGIMIEB) */
#define _00_TMRG_INTERRUPT_TRGIMFB_DISABLE          (0x00U)             /* interrupt by TRGIMFB bit disabled */
#define _02_TMRG_INTERRUPT_TRGIMFB_ENABLE           (0x02U)             /* interrupt by TRGIMFB bit enabled */
/* Input-capture/compare-match interrupt enable A (TRGIMIEA) */
#define _00_TMRG_INTERRUPT_TRGIMFA_DISABLE          (0x00U)             /* interrupt by TRGIMFA bit disabled */
#define _01_TMRG_INTERRUPT_TRGIMFA_ENABLE           (0x01U)             /* interrupt by TRGIMFA bit enabled */

/*
    Timer RG I/O control resister (TRGIOR) 
*/
/* TRGGRD register function select (TRGBUFB) */
#define _00_TMRG_TRGGRB_REGISTER_UNUSED             (0x00U)             /* not used as buffer register for TRGGRB register */
#define _80_TMRG_TRGGRB_REGISTER_USED               (0x80U)             /* used as buffer register for TRGGRB register */
/* TRGGRB mode select (TRGIOB2) */
#define _00_TMRG_TRGGRB_OUTPUT_COMPARE              (0x00U)             /* output compare function */
#define _40_TMRG_TRGGRB_INPUT_CAPTURE               (0x40U)             /* input capture function */
/* TRGGRB control (TRGIOB1,TRGIOB0) */
#define _00_TMRG_TRGGRB_OUTPUT_DISABLED             (0x00U)             /* pin output by compare match is disabled */
#define _10_TMRG_TRGGRB_LOW_OUTPUT                  (0x10U)             /* low output */
#define _20_TMRG_TRGGRB_HIGH_OUTPUT                 (0x20U)             /* high output */
#define _30_TMRG_TRGGRB_TOGGLE_OUTPUT               (0x30U)             /* toggle output */
/* TRGGRB control (TRGIOB1,TRGIOB0) */
#define _00_TMRG_TRGGRB_RISING_EDGE                 (0x00U)             /* rising edge of TRGIOB */
#define _10_TMRG_TRGGRB_FALLING_EDGE                (0x10U)             /* falling edge of TRGIOB */
#define _20_TMRG_TRGGRB_BOTH_EDGES                  (0x20U)             /* both edges of TRGIOB */
/* TRGGRC register function select (TRGBUFA) */
#define _00_TMRG_TRGGRA_REGISTER_UNUSED             (0x00U)             /* not used as buffer register for TRGGRA register */
#define _08_TMRG_TRGGRA_REGISTER_USED               (0x08U)             /* used as buffer register for TRGGRA register */
/* TRGGRA mode select (TRGIOA2) */
#define _00_TMRG_TRGGRA_OUTPUT_COMPARE              (0x00U)             /* output compare function */
#define _04_TMRG_TRGGRA_INPUT_CAPTURE               (0x04U)             /* input capture function */
/* TRGGRA control (TRGIOA1,TRGIOA0) */
#define _00_TMRG_TRGGRA_OUTPUT_DISABLED             (0x00U)             /* pin output by compare match is disabled */
#define _01_TMRG_TRGGRA_LOW_OUTPUT                  (0x01U)             /* low output */
#define _02_TMRG_TRGGRA_HIGH_OUTPUT                 (0x02U)             /* high output */
#define _03_TMRG_TRGGRA_TOGGLE_OUTPUT               (0x03U)             /* toggle output */


/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/
#define _0000_TMRG_TRG_VALUE                        (0x0000U)           /* timer RG counter */

/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/
typedef enum
{
    TMRG_CHANNELA,
    TMRG_CHANNELB,
    TMRG_CHANNELELC
} tmrg_channel_t;

/***********************************************************************************************************************
Global functions
***********************************************************************************************************************/
void R_TMRG0_Create(void);
void R_TMRG0_Start(void);
void R_TMRG0_Stop(void);
/* Start user code for function. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#endif
