/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2017-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_calc_encd_foc.h
* Description : A definitions of the processing of a calculation layer
******************************************************************************/
/******************************************************************************
* History : 29.05.2017  Ver.1.00
******************************************************************************/

/* guard against multiple inclusion */
#ifndef MTR_CALC_ENCD_FOC_H
#define MTR_CALC_ENCD_FOC_H

/******************************************************************************
* Macro definitions
******************************************************************************/
#ifndef abs
#define abs(a)                  (((a) > 0) ? (a) : -(a))
#endif

#define MTR_ENCD_CPR_MECH       (1200)                                  /* TRGGRA + 1 */

#define MTR_ENCD_POLE_PAIRS     (MP_POLE_PAIRS)                         /* number of pole pairs */

#define MTR_ENCD_STATE_ALIGN    (0)                                     /* calculate the theta from the counter(alignment) */
#define MTR_ENCD_STATE_INIT     (1)                                     /* calculate the theta from the counter(initial position) */
#define MTR_ENCD_STATE_CURRENT  (2)                                     /* calculate the theta from the timer RG(current control) */
#define MTR_ENCD_STATE_SPEED    (3)                                     /* calculate the theta from the timer RG(speed control) */
#define MTR_ENCD_STATE_POSITION (4)                                     /* calculate the theta from the timer RG(position control) */

#define MTR_ENCD_STATE_DEFAULT  (MTR_ENCD_STATE_POSITION)               /* default state */

#define MTR_COEF_ENCD_THETA     (3.413333333f)                          /* 4096 / 1200 = ((PI() * 2) / 1200) * (4096 / (PI() * 2)) */
#define MTR_COEF_ENCD_OMEGA     (5.235987756f)                          /* (PI() * 2) / 1200 / 1[ms] */

#define MTR_COEF_LPF1           (0.7f)                                  /* Coefficient1 of the LPF */
#define MTR_COEF_LPF2           (0.3f)                                  /* Coefficient2 of the LPF */

#define MTR_COEF_AMP            (20.01955034f)                          /* 5 / 1023 * 4096 */

#define MTR_COEF_G              (0.1f)                                  /* coefficient of converting voltage to current */

#define MTR_COEF_K              (2216.684724f)                          /* 100 * 1023 / 5 / 9.23 */

#define MTR_CENTER_AMP          (10240)                                 /* 5 / 2 * 4096 */

#define MTR_CENTER_ADC          (512)                                   /* 1023 / 2 : scale(Q0) */

#define MTR_CENTER_PWM_DUTY     (50)                                    /* 50% : scale(Q0) */

#define MTR_FIXED_THETA         (585)                                   /* fixed value : scale(Q0: 0 to 4095) */
#define MTR_FIXED_OMEGA         (0)                                     /* fixed value(0.0) : scale(Q4) */
#define MTR_FIXED_ID            (20480)                                 /* fixed value(5.0) : scale(Q12) */
#define MTR_FIXED_IQ            (0)                                     /* fixed value(0.0) : scale(Q12) */

#define MTR_LIMIT_ADC           (1023U)                                 /* limit value : scale(Q0) */
#define MTR_LIMIT_THETA         (4095U)                                 /* limit value : scale(Q0: 0 to 4095) */

#define MTR_SRL_THETA           (10U)                                   /* slew rate limit value : scale(Q0: 0 to 4095) */
#define MTR_SRL_OMEGA           (10U)                                   /* slew rate limit value(0.625) : scale(Q4) */

/* PID(common) */
#define MTR_PID_LIMIT_ZI        (4000)                                  /* limit value */

#define MTR_PID_THETA_TH        (100)                                   /* threshold for switching parameters : scale(Q0: 0 to 4095) */

/* PID(theta) */
#define MTR_PID_THETA_KP        (100.0f)
#define MTR_PID_THETA_TI        (0.0005f)                               /* 1[ms] / 2 */
#define MTR_PID_THETA_KI        (0.3f)
#define MTR_PID_THETA_TD        (1000.0f)                               /* 1 / 1[ms] */
#define MTR_PID_THETA_KD        (0.001f)
#define MTR_PID_THETA_KT        (0.004f)

#define MTR_PID_THETA_KP_H      (160.0f)

#define MTR_PID_THETA_KT_H      (0.005f)

/* PID(omega) */
#define MTR_PID_OMEGA_KP        (1.0f)
#define MTR_PID_OMEGA_TI        (0.0005f)                               /* 1[ms] / 2 */
#define MTR_PID_OMEGA_KI        (0.01f)
#define MTR_PID_OMEGA_TD        (1000.0f)                               /* 1 / 1[ms] */
#define MTR_PID_OMEGA_KD        (0.001f)
#define MTR_PID_OMEGA_KT        (0.075f)

#define MTR_PID_OMEGA_KT_H      (0.1f)

/* PID(id) */
#define MTR_PID_ID_KP           (0.5f)
#define MTR_PID_ID_TI           (0.0005f)                               /* 1[ms] / 2 */
#define MTR_PID_ID_KI           (0.0f)
#define MTR_PID_ID_TD           (1000.0f)                               /* 1 / 1[ms] */
#define MTR_PID_ID_KD           (0.0f)
#define MTR_PID_ID_KT           (0.26f)

/* PID(iq) */
#define MTR_PID_IQ_KP           (1000.0f)
#define MTR_PID_IQ_TI           (0.0005f)                               /* 1[ms] / 2 */
#define MTR_PID_IQ_KI           (0.1f)
#define MTR_PID_IQ_TD           (1000.0f)                               /* 1 / 1[ms] */
#define MTR_PID_IQ_KD           (0.0f)
#define MTR_PID_IQ_KT           (0.0013f)

typedef struct
{
    float32_t  f4_kp;                                                   /* proportion gain */
    float32_t  f4_ti;                                                   /* integral period */
    float32_t  f4_ki;                                                   /* integral gain */
    float32_t  f4_td;                                                   /* differential period */
    float32_t  f4_kd;                                                   /* differential gain */
    float32_t  f4_kt;                                                   /* total gain */
    float32_t *f4_zi;                                                   /* integral delay */
    float32_t *f4_zd;                                                   /* differential delay */
} MTR_PID_PRM;                                                          /* PID Parameters */

/******************************************************************************
* Global function definitions
******************************************************************************/
void mtr_calc_InitVariables(void);                                      /* initialize private global variables */

void mtr_calc_PID(int16_t, int16_t *, MTR_PID_PRM);                     /* calculate Proportion, Integral and Differential */

int16_t mtr_calc_SRL(int16_t, int16_t *, uint16_t);                     /* calculate Slew Rate Limiter */

void mtr_calc_EncoderInput(int16_t *, int16_t *, int16_t *);            /* encoder input */

#endif /* MTR_CALC_ENCD_FOC_H */
