/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_ctrl_rl78g1f_t2001.h
* Description : A definitions of the processing of a H/W control layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/* guard against multiple inclusion */
#ifndef MTR_CTRL_RL78G1F_T2001_H
#define MTR_CTRL_RL78G1F_T2001_H

/******************************************************************************
* Macro definitions
******************************************************************************/
#define MTR_PWM_TIMER_FREQ      (64.0f)                                 /* PWM timer frequency[MHz] */
#define MTR_CARRIER_FREQ        (20.0f)                                 /* carrier wave frequency[KHz] */

#define MTR_DEADTIME            (0)                                     /* deadtime[ns] */
#define MTR_DEADTIME_SET        ((int16_t)((MTR_DEADTIME * MTR_PWM_TIMER_FREQ) / 1000))
                                                                        /* setting value of deadtime */
#define MTR_CARRIER_SET         ((((MTR_PWM_TIMER_FREQ * 1000) / MTR_CARRIER_FREQ) / 2) + MTR_DEADTIME_SET - 2)
                                                                        /* setting value of carrier */
#define MTR_START_CARRIER_SET   ((MTR_CARRIER_SET * 50) / 100)          /* setting value of 50% carrier */

#define MTR_VR1_ADC_MAX         (802)                                   /* max valule of VR1 ADC */

#define MTR_THETA_CALC_COEF     (4095.0f / MTR_VR1_ADC_MAX)             /* coef of theta calculation */
#define MTR_OMEGA_CALC_COEF     (MTR_TWOPI / 60.0f * 16.0f)             /* coef of omega calculation */

#define MTR_RPM_CALC_COEF       (1221.7f / MTR_VR1_ADC_MAX)             /* coef of rpm calculation */
#define MTR_GET_ROT_DIR_REQ     (P12.2)                                 /* get Rotation Direction Request */

#define MTR_PORT_LED3           (P14.1)                                 /* output port of LED3 */
#define MTR_PORT_LED4           (P14.0)                                 /* output port of LED4 */
#define MTR_PORT_LED5           (P4.3)                                  /* output port of LED5 */
#define MTR_PORT_LED6           (P4.2)                                  /* output port of LED6 */

#define MTR_LED_ON              (0)                                     /* active level of LED */
#define MTR_LED_OFF             (1)                                     /* inactive level of LED */

#define MTR_OVERVOLTAGE_LIMIT   ((int16_t)(30 * 128))                   /* over voltage limit [V] */
#define MTR_UNDERVOLTAGE_LIMIT  ((int16_t)( 6 * 128))                   /* under voltage limit [V] */

#define MTR_VDC_SCALING         (1471)                                  /* calculation factor (bus voltage) */

/* A/D converter channel number */
#define MTR_ADCCH_RAJ306000_TEMP (0)                                    /* temperature of RAJ306000 */
#define MTR_ADCCH_MOS_TEMP       (1)                                    /* temperature of MOS */
#define MTR_ADCCH_IU             (2)                                    /* U-phase voltage */
#define MTR_ADCCH_VM             (3)                                    /* voltage of VM */
#define MTR_ADCCH_IW             (5)                                    /* W-phase voltage */
#define MTR_ADCCH_VR1            (6)                                    /* VR1 */
#define MTR_ADCCH_PDRV           (7)                                    /* voltage of prideriver */

/******************************************************************************
* Global function definitions
******************************************************************************/
void mtr_ctrl_start(void);                                              /* motor control start */
void mtr_ctrl_stop(void);                                               /* motor control stop */
void mtr_change_pattern(void);                                          /* change voltage pattern */

int16_t mtr_get_adc(uint8_t);                                           /* A/D conversion */

#endif /* MTR_CTRL_RL78G1F_T2001_H */
