/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_ssns_encd_foc.h
* Description : A definitions of the processing of a motor control layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/* guard against multiple inclusion */
#ifndef MTR_SSNS_ENCD_FOC_H
#define MTR_SSNS_ENCD_FOC_H

/******************************************************************************
* Macro definitions
******************************************************************************/
#define MTR_TWOPI               (2 * 3.14159265f)                       /* 2 * pi */

#define MTR_SPEED_LIMIT_RPM     (1000)                                  /* over speed limit [rpm] (mechanical angle) : scale(2^0) */
#define MTR_SPEED_LIMIT         ((int16_t)((MTR_SPEED_LIMIT_RPM / 60.0f) * MTR_TWOPI * 16.0f))
                                                                        /* over speed limit [rad/s] (mechanical angle) : scale(2^4) */
#define MTR_TIMEOUT_CNT         (400)                                   /* undetected time = MTR_TIMEOUT_CNT * 50[us] */

/* rotational direction */
#define MTR_CW                  (0)
#define MTR_CCW                 (1)

#define MTR_FLG_CLR             (0)                                     /* for flag clear */
#define MTR_FLG_SET             (1)                                     /* for flag set */
#define MTR_STOP_WAIT_CNT       (200)
/* Timer RD interrupt period(50[us]) * MTR_STOP_WAIT_CNT = motor stop waiting time */

#define MTR_ICS_DECIMATION      (4)                                     /* ICS decimation count */

/******************************************************************************
* Macro definitions for sequence control
******************************************************************************/
/* error status */
#define MTR_ALARM_ERROR         (0x01)
#define MTR_OVER_VOLTAGE_ERROR  (0x02)
#define MTR_OVER_SPEED_ERROR    (0x04)
#define MTR_TIMEOUT_ERROR       (0x08)

#define MTR_BEMF_ERROR          (0x20)
#define MTR_UNDER_VOLTAGE_ERROR (0x40)
#define MTR_UNKNOWN_ERROR       (0x80)
#define MTR_HW_ERROR            (MTR_ALARM_ERROR)
#define MTR_FW_ERROR            (MTR_OVER_VOLTAGE_ERROR | MTR_OVER_SPEED_ERROR    | MTR_TIMEOUT_ERROR |\
                                                          MTR_UNDER_VOLTAGE_ERROR | MTR_UNKNOWN_ERROR)

/* system mode */
#define MTR_MODE_STOP           (0x00)
#define MTR_MODE_RUN            (0x01)
#define MTR_MODE_ERROR          (0x02)
#define MTR_SIZE_STATE          (3)                                     /* number of state */

/* event */
#define MTR_EVENT_STOP          (0x00)
#define MTR_EVENT_RUN           (0x01)
#define MTR_EVENT_ERROR         (0x02)
#define MTR_EVENT_RESET         (0x03)
#define MTR_SIZE_EVENT          (4)                                     /* number of event */

typedef struct
{
    int16_t    s2_theta;
    int16_t    s2_omega;
} MTR_ICS_INPUT;

/******************************************************************************
* Global function definitions
******************************************************************************/
void R_MTR_IcsInput(MTR_ICS_INPUT *);                                   /* set value to buffer */
void R_MTR_SetSpeed(int16_t);                                           /* set speed reference */
void R_MTR_SetDir(uint8_t);                                             /* set rotational direction */
uint8_t R_MTR_GetDir(void);                                             /* get rotation direction */
uint8_t R_MTR_GetStatus(void);                                          /* get status of motor control system */

void mtr_pattern_set(void);                                             /* set voltage pattern */
void mtr_set_variables(void);                                           /* set variables (conversion of unit) */
void mtr_error_check(void);                                             /* check error status */

/* for sequence control */
void R_MTR_InitSequence(void);                                          /* initialize variables for sequence control */
void R_MTR_ExecEvent(uint8_t);                                          /* change motor state and select action */

/* for encoder vector control */
void R_MTR_InitEncoderVector(void);                                     /* initialize variables for encoder vector control */
void R_MTR_SetFlagCtrlDuty(uint16_t);                                   /* set/clear a flag to control PWM duty */

#endif /* MTR_SSNS_ENCD_FOC_H */
