/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_ctrl_rl78g1f_t2001.c
* Description : The processing of a H/W control layer(MCU dependance)
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"

/******************************************************************************
* Exported global variables
******************************************************************************/
/* for encoder vector control */
extern volatile uint16_t   g_u2_foc_ud;                                 /* U phase PWM duty : scale(Q0) */
extern volatile uint16_t   g_u2_foc_vd;                                 /* V phase PWM duty : scale(Q0) */
extern volatile uint16_t   g_u2_foc_wd;                                 /* W phase PWM duty : scale(Q0) */

extern volatile uint16_t   g_u2_foc_flg_ctrl_duty;                      /* flag to control PWM duty */

/******************************************************************************
* Function Name : mtr_ctrl_start
* Description   : motor control start (set a flag to control PWM duty)
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_ctrl_start(void)
{
    g_u2_foc_flg_ctrl_duty = 1;                                         /* MTR_FLG_SET */
}

/******************************************************************************
* Function Name : mtr_ctrl_stop
* Description   : motor control stop (clear a flag to control PWM duty)
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_ctrl_stop(void)
{
    g_u2_foc_flg_ctrl_duty = 0;                                         /* MTR_FLG_CLR */
}

/******************************************************************************
* Function Name : mtr_change_pattern
* Description   : Change voltage pattern
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_change_pattern(void)
{
    /*========================*/
    /*     stop PWM timer     */
    /*========================*/

    TRDSTR = 0x0C;                                                      /* stop TRD0 */

    /*======================*/
    /*     set register     */
    /*======================*/

    TRDOER1 = 0x05;                                                     /* TRD output enable */

    if (g_u2_foc_flg_ctrl_duty == 0)                                    /* MTR_FLG_CLR */
    {
        g_u2_foc_ud = (uint16_t)MTR_START_CARRIER_SET;
        g_u2_foc_vd = (uint16_t)MTR_START_CARRIER_SET;
        g_u2_foc_wd = (uint16_t)MTR_START_CARRIER_SET;
    }

    TRDGRD0 = g_u2_foc_ud;                                              /* set U phase PWM duty */
    TRDGRC1 = g_u2_foc_vd;                                              /* set V phase PWM duty */
    TRDGRD1 = g_u2_foc_wd;                                              /* set W phase PWM duty */
    TRDGRB0 = g_u2_foc_ud;                                              /* set U phase PWM duty */
    TRDGRA1 = g_u2_foc_vd;                                              /* set V phase PWM duty */
    TRDGRB1 = g_u2_foc_wd;                                              /* set W phase PWM duty */

    /*===========================*/
    /*     restart PWM timer     */
    /*===========================*/

    TRD0 = (uint16_t)MTR_DEADTIME_SET;
    TRD1 = 0;

    TRDSTR = 0x0F;                                                      /* start TRD0 */
}

/******************************************************************************
* Function Name : mtr_get_adc
* Description   : A/D conversion
* Arguments     : ad_ch - A/D converter channel number
* Return Value  : A/D converted value
******************************************************************************/
int16_t mtr_get_adc(uint8_t ad_ch)
{
    int16_t s2_temp;

    ADS = ad_ch;                                                        /* set converted channel */

    ADCS = 1;

    while (ADIF == 0)
    {
        /* waiting for A/D converter */
    }

    s2_temp = ((ADCR >> 6) & 0x03FF);                                   /* get A/D converted value (10bit) */

    ADIF = 0;

    return (s2_temp);
}

