/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_ctrl_t2001.c
* Description : The processing of a H/W control layer(Inverter board dependance)
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "mtr_ctrl_t2001.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"

/******************************************************************************
* Function Name : get_vr1
* Description   : Get A/D converted value of VR1
* Arguments     : none
* Return Value  : A/D converted value of VR1
******************************************************************************/
uint16_t get_vr1(void)
{
    uint16_t ad_data;

    DI();

    ad_data = mtr_get_adc(MTR_ADCCH_VR1);

    EI();

    return (ad_data);
}

/******************************************************************************
* Function Name : led_on
* Description   : Turn on LED
* Arguments     : LED channel number
* Return Value  : none
******************************************************************************/
void led_on(uint8_t ch)
{
#ifdef LED_CTRL_ON
    switch (ch)
    {
        case MTR_LED3:
            MTR_PORT_LED3 = MTR_LED_ON;
        break;

        case MTR_LED4:
            MTR_PORT_LED4 = MTR_LED_ON;
        break;

        case MTR_LED5:
            MTR_PORT_LED5 = MTR_LED_ON;
        break;

        case MTR_LED6:
            MTR_PORT_LED6 = MTR_LED_ON;
        break;

        default:
        break;
    }
#endif
}

/******************************************************************************
* Function Name : led_off
* Description   : Turn off LED
* Arguments     : LED channel number
* Return Value  : none
******************************************************************************/
void led_off(uint8_t ch)
{
#ifdef LED_CTRL_ON
    switch (ch)
    {
        case MTR_LED3:
            MTR_PORT_LED3 = MTR_LED_OFF;
        break;

        case MTR_LED4:
            MTR_PORT_LED4 = MTR_LED_OFF;
        break;

        case MTR_LED5:
            MTR_PORT_LED5 = MTR_LED_OFF;
        break;

        case MTR_LED6:
            MTR_PORT_LED6 = MTR_LED_OFF;
        break;

        default:
        break;
    }
#endif
}

