/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_interrupt.c
* Description : The interrupt handler
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Interrupt vector define
******************************************************************************/
#pragma interrupt INTP4   mtr_alarm_interrupt
#pragma interrupt INTTM00 mtr_tau0_interrupt
#pragma interrupt INTTM02 mtr_tau2_interrupt
#pragma interrupt INTTRD0 mtr_carrier_interrupt

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "control_parameter.h"
#include "motor_parameter.h"
#include "mtr_ssns_encd_foc.h"
#include "mtr_calc_encd_foc.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"
#include "r_cg_tau.h"

#ifdef ICS_USE
/****** for ICS ******/
#include "ics_RL78G1F_Lx.h"
/*********************/
#endif

/******************************************************************************
* Exported global variables
******************************************************************************/
extern volatile uint16_t   g_u2_cnt_wait_stop;                          /* counter for waiting for motor stop */
extern volatile uint8_t    g_u1_flg_wait_stop;                          /* flag for waiting for motor stop */

extern volatile uint8_t    g_u1_enable_write;                           /* ICS write enable flag */

#ifdef ICS_USE
/****** for ICS ******/
extern volatile uint8_t    g_u1_cnt_ics;                                /* counter for period of calling "scope_watchpoint" */
/*********************/
#endif

/* for sequence control */
extern volatile uint8_t    g_u1_error_status;                           /* error status */

/* for encoder vector control */
extern volatile int16_t    g_s2_foc_encd_status;                        /* encoder vector control status */

extern volatile int16_t    g_s2_foc_ref_theta;                          /* reference angle [rad] (normalization) : scale(Q0: 0 to 4095) */
extern volatile int16_t    g_s2_foc_ref_omega;                          /* reference speed [rad/s] : scale(Q4) */

extern volatile int16_t    g_s2_foc_theta;                              /* angle [rad] (normalization) : scale(Q0: 0 to 4095) */
extern volatile int16_t    g_s2_foc_theta_elec;
extern volatile int16_t    g_s2_foc_theta_mech;

extern volatile int16_t    g_s2_foc_theta_zs;                           /* delay for SRL(theta) */

extern volatile MTR_PID_PRM pid_foc_theta;                              /* struct for PID(theta) Parameters */

extern volatile int16_t    g_s2_foc_omega;                              /* speed [rad/s] : scale(Q4) */
extern volatile int16_t    g_s2_foc_omega_mech;

extern volatile int16_t    g_s2_foc_omega_zs;                           /* delay for SRL(omega) */

extern volatile MTR_PID_PRM pid_foc_omega;                              /* struct for PID(omega) Parameters */

extern volatile int16_t    g_s2_foc_id;                                 /* rotating coordinate id : scale(Q12) */

extern volatile MTR_PID_PRM pid_foc_id;                                 /* struct for PID(id) Parameters */

extern volatile int16_t    g_s2_foc_iq;                                 /* rotating coordinate iq : scale(Q12) */

extern volatile MTR_PID_PRM pid_foc_iq;                                 /* struct for PID(iq) Parameters */

extern volatile int16_t    g_s2_foc_ia;                                 /* stationary coordinate ia : scale(Q12) */
extern volatile int16_t    g_s2_foc_ib;                                 /* stationary coordinate ib : scale(Q12) */

extern volatile int16_t    g_s2_foc_ui;                                 /* U phase current : scale(Q12) */
extern volatile int16_t    g_s2_foc_vi;                                 /* V phase current : scale(Q12) */
extern volatile int16_t    g_s2_foc_wi;                                 /* W phase current : scale(Q12) */

extern volatile float32_t  g_f4_foc_coef_k;                             /* coefficient(ratio) of converting current to voltage */

extern volatile int16_t    g_s2_foc_uv;                                 /* U phase voltage(PWM duty) : scale(Q0) */
extern volatile int16_t    g_s2_foc_vv;                                 /* V phase voltage(PWM duty) : scale(Q0) */
extern volatile int16_t    g_s2_foc_wv;                                 /* W phase voltage(PWM duty) : scale(Q0) */

extern volatile uint16_t   g_u2_foc_ud;                                 /* U phase PWM duty(setting value) : scale(Q0) */
extern volatile uint16_t   g_u2_foc_vd;                                 /* V phase PWM duty(setting value) : scale(Q0) */
extern volatile uint16_t   g_u2_foc_wd;                                 /* W phase PWM duty(setting value) : scale(Q0) */

extern volatile int16_t    g_s2_foc_uad_offset;                         /* U phase voltage of shunt resistor(offset) : scale(Q0) */
extern volatile int16_t    g_s2_foc_wad_offset;                         /* W phase voltage of shunt resistor(offset) : scale(Q0) */

extern volatile int16_t    g_s2_foc_uad;                                /* U phase voltage of shunt resistor : scale(Q0) */
extern volatile int16_t    g_s2_foc_wad;                                /* W phase voltage of shunt resistor : scale(Q0) */

extern volatile int16_t    g_s2_foc_adc_toggle;                         /* toggle switch for A/D conversion */

extern volatile int16_t    g_s2_foc_uadd;                               /* uad' : scale(Q12) */
extern volatile int16_t    g_s2_foc_wadd;                               /* wad' : scale(Q12) */

extern volatile int16_t    g_s2_foc_iad;                                /* ia' : scale(Q12) */
extern volatile int16_t    g_s2_foc_ibd;                                /* ib' : scale(Q12) */

extern volatile int16_t    g_s2_foc_iadd;                               /* ia'' : scale(Q12) */
extern volatile int16_t    g_s2_foc_ibdd;                               /* ib'' : scale(Q12) */

extern volatile int16_t    g_s2_foc_idd;                                /* id' : scale(Q12) */
extern volatile int16_t    g_s2_foc_iqd;                                /* iq' : scale(Q12) */

/******************************************************************************
* Function Name : mtr_alarm_interrupt
* Description   : Alarm interrupt(INTP4)
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_alarm_interrupt(void)
{
#ifdef HW_ERR_DET_ON
    R_MTR_ExecEvent(MTR_EVENT_ERROR);                                   /* execute error event */

    DI();

    g_u1_error_status |= MTR_ALARM_ERROR;                               /* error status update : alarm error */

    EI();

    mtr_clear_oc_flag();                                                /* clear forced cutoff flag (detect alarm) */
#endif
}

/******************************************************************************
* Function Name : mtr_tau0_interrupt
* Description   : TAU0 channel 0 interrupt (period: 1[ms])
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_tau0_interrupt(void)
{
    EI();

    /* calculate the input from the encoder */
    mtr_calc_EncoderInput(&g_s2_foc_theta_elec, &g_s2_foc_theta_mech, &g_s2_foc_omega_mech);

    /* Normalize the A/D conversion value */                            /* convert scale(Q0 -> Q12) */
    g_s2_foc_uadd = (int16_t)((float32_t)g_s2_foc_uad * MTR_COEF_AMP) - MTR_CENTER_AMP;
    g_s2_foc_wadd = (int16_t)((float32_t)g_s2_foc_wad * MTR_COEF_AMP) - MTR_CENTER_AMP;

    /* Clarke transformation */
    R_motor_uw2ab_int16(g_s2_foc_uadd, g_s2_foc_wadd, &g_s2_foc_iad, &g_s2_foc_ibd);

    g_s2_foc_iadd = (int16_t)(MTR_COEF_G * (float32_t)g_s2_foc_iad);
    g_s2_foc_ibdd = (int16_t)(MTR_COEF_G * (float32_t)g_s2_foc_ibd);

    /* Park transformation */
    R_motor_ab2dq_int16(g_s2_foc_iadd, g_s2_foc_ibdd, g_s2_foc_theta_elec, &g_s2_foc_idd, &g_s2_foc_iqd);

    /* encoder vector control(position control) */
    if (g_s2_foc_encd_status == MTR_ENCD_STATE_POSITION)
    {
        /* calculate the SRL(theta) */
        g_s2_foc_theta = mtr_calc_SRL(g_s2_foc_ref_theta, &g_s2_foc_theta_zs, MTR_SRL_THETA);

        g_s2_foc_theta -= g_s2_foc_theta_mech;

        g_s2_foc_theta = R_FIX_limit_int16(g_s2_foc_theta, MTR_LIMIT_THETA);

        if (abs(g_s2_foc_theta) < MTR_PID_THETA_TH)
        {
            pid_foc_theta.f4_kp = MTR_PID_THETA_KP_H;
            pid_foc_theta.f4_kt = MTR_PID_THETA_KT_H;

            pid_foc_omega.f4_kt = MTR_PID_OMEGA_KT_H;
        }
        else
        {
            pid_foc_theta.f4_kp = MTR_PID_THETA_KP;
            pid_foc_theta.f4_kt = MTR_PID_THETA_KT;

            pid_foc_omega.f4_kt = MTR_PID_OMEGA_KT;
        }

        /* PID(theta) */
        g_s2_foc_ref_omega = 0;

        /* calculate the PID(theta) */
        mtr_calc_PID(g_s2_foc_theta, &g_s2_foc_ref_omega, pid_foc_theta);
    }

    /* encoder vector control(speed control) */
    if (g_s2_foc_encd_status >= MTR_ENCD_STATE_SPEED)
    {
        /* calculate the SRL(omega) */
        g_s2_foc_omega = mtr_calc_SRL(g_s2_foc_ref_omega, &g_s2_foc_omega_zs, MTR_SRL_OMEGA);

        g_s2_foc_omega -= g_s2_foc_omega_mech;

        /* calculate the PID(omega) */
        mtr_calc_PID(g_s2_foc_omega, &g_s2_foc_iq, pid_foc_omega);

        g_s2_foc_iq *= 256;                                             /* convert scale(Q4 -> Q12) */
    }

    if (g_s2_foc_encd_status >= MTR_ENCD_STATE_SPEED)
    {
        g_s2_foc_id = (-g_s2_foc_idd);
    }
    else
    {
        g_s2_foc_id = MTR_FIXED_ID;
    }

    /* calculate the PID(id) */
    mtr_calc_PID(g_s2_foc_id, &g_s2_foc_id, pid_foc_id);

    if (g_s2_foc_encd_status >= MTR_ENCD_STATE_SPEED)
    {
        g_s2_foc_iq = g_s2_foc_iq - g_s2_foc_iqd;
    }
    else
    {
        g_s2_foc_iq = MTR_FIXED_IQ;
    }

    /* calculate the PID(iq) */
    mtr_calc_PID(g_s2_foc_iq, &g_s2_foc_iq, pid_foc_iq);

    /* inverse Park transformation */
    R_motor_dq2ab_int16(g_s2_foc_id, g_s2_foc_iq, g_s2_foc_theta_elec, &g_s2_foc_ia, &g_s2_foc_ib);

    /* inverse Clarke transformation */
    R_motor_ab2uvw_int16(g_s2_foc_ia, g_s2_foc_ib, &g_s2_foc_ui, &g_s2_foc_vi, &g_s2_foc_wi);

    /* inverter output */                                               /* convert scale(Q12 -> Q0) */
    g_s2_foc_uv = MTR_CENTER_PWM_DUTY + (int16_t)(g_f4_foc_coef_k * (float32_t)g_s2_foc_ui / 4096.0f);
    g_s2_foc_vv = MTR_CENTER_PWM_DUTY + (int16_t)(g_f4_foc_coef_k * (float32_t)g_s2_foc_vi / 4096.0f);
    g_s2_foc_wv = MTR_CENTER_PWM_DUTY + (int16_t)(g_f4_foc_coef_k * (float32_t)g_s2_foc_wi / 4096.0f);

    /* calculate the output to the motor */
    g_u2_foc_ud = (uint16_t)(MTR_CARRIER_SET * (float32_t)g_s2_foc_uv / 100.0f);
    g_u2_foc_vd = (uint16_t)(MTR_CARRIER_SET * (float32_t)g_s2_foc_vv / 100.0f);
    g_u2_foc_wd = (uint16_t)(MTR_CARRIER_SET * (float32_t)g_s2_foc_wv / 100.0f);

    mtr_pattern_set();                                                  /* set voltage pattern */

    DI();
}

/******************************************************************************
* Function Name : mtr_tau2_interrupt
* Description   : TAU0 channel 2 interrupt (Period: 25[us])
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_tau2_interrupt(void)
{
    int16_t s2_temp;

    EI();

    R_TAU0_Channel2_Stop();                                             /* TAU0 channel 2 count stop */

    switch (g_s2_foc_adc_toggle)
    {
        case MTR_ADCCH_IU: 

            DI();

            s2_temp = mtr_get_adc(MTR_ADCCH_IU);

            EI();

            g_s2_foc_uad = (int16_t)R_FIX_ulimit_int16((s2_temp + g_s2_foc_uad_offset), MTR_LIMIT_ADC);

            g_s2_foc_adc_toggle = MTR_ADCCH_IW;
        break;

        case MTR_ADCCH_IW: 

            DI();

            s2_temp = mtr_get_adc(MTR_ADCCH_IW);

            EI();

            g_s2_foc_wad = (int16_t)R_FIX_ulimit_int16((s2_temp + g_s2_foc_wad_offset), MTR_LIMIT_ADC);

            g_s2_foc_adc_toggle = MTR_ADCCH_IU;
        break;

        default:
        break;
    }

    DI();
}

/******************************************************************************
* Function Name : mtr_carrier_interrupt
* Description   : Carrier interrupt (period: 50[us])
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_carrier_interrupt(void)
{

    EI();

    R_TAU0_Channel2_Start();                                            /* TAU0 channel 2 count start */

    /*============================*/
    /*      check error           */
    /*============================*/

    mtr_error_check();                                                  /* check error status */

    /*** wait motor stop ***/
    if (g_u1_flg_wait_stop == MTR_FLG_SET)                              /* waiting for motor stop */
    {
        g_u2_cnt_wait_stop++;

        if (g_u2_cnt_wait_stop >= MTR_STOP_WAIT_CNT)
        {
            g_u1_flg_wait_stop = MTR_FLG_CLR;
       }
    }

#ifdef ICS_USE
    /****** for ICS ******/
    g_u1_cnt_ics++;

    if (g_u1_cnt_ics > MTR_ICS_DECIMATION)                              /* period : 250[us] */
    {
        DI();                                                           /* interrupt disable */

        RPECTL.7 = 1;
        ics_watchpoint();
        RPECTL.7 = 0;

        EI();                                                           /* interrupt enable */

        g_u1_cnt_ics = 0;
    }
    /*********************/
#endif

    /*** set variables ***/
    if (MTR_FLG_SET == g_u1_enable_write)
    {
        mtr_set_variables();
        g_u1_enable_write = MTR_FLG_CLR;
    }

    mtr_clear_trd0_imfa();                                              /* clear compare match flag A */

    DI();
}

