/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_ssns_encd_foc.c
* Description : The processing of a motor control layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "control_parameter.h"
#include "motor_parameter.h"
#include "mtr_ssns_encd_foc.h"
#include "mtr_calc_encd_foc.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"

/******************************************************************************
* Exported global variables (to be accessed by other files)
******************************************************************************/
volatile uint16_t   g_u2_cnt_wait_stop;                                 /* counter for waiting for motor stop */
volatile uint8_t    g_u1_flg_wait_stop;                                 /* flag for waiting for motor stop */

volatile uint8_t    g_u1_enable_write;                                  /* ICS write enable flag */

volatile int16_t    g_s2_vdc_ad;                                        /* bus voltage : scale(2^7) */
volatile int16_t    g_s2_pdrv_ad;                                       /* predriver voltage : scale(2^0) */

#ifdef ICS_USE
/****** for ICS ******/
volatile uint8_t    g_u1_cnt_ics;                                       /* counter for period of calling "scope_watchpoint" */
/*********************/
#endif

/* for sequence control */
volatile uint8_t    g_u1_error_status;                                  /* error status */
volatile uint8_t    g_u1_mode_system;                                   /* system mode */

/* for encoder vector control */
volatile int16_t    g_s2_foc_encd_status;                               /* encoder vector control status */

volatile int16_t    g_s2_foc_mech_count;                                /* count value(mechanical) */
volatile int16_t    g_s2_foc_elec_count;                                /* count value(electrical) */

volatile int16_t    g_s2_foc_ref_theta;                                 /* reference angle [rad] (normalization) : scale(Q0: 0 to 4095) */
volatile int16_t    g_s2_foc_ref_omega;                                 /* reference speed [rad/s] : scale(Q4) */

volatile int16_t    g_s2_foc_theta;                                     /* angle [rad] (normalization) : scale(Q0: 0 to 4095) */
volatile int16_t    g_s2_foc_theta_elec;
volatile int16_t    g_s2_foc_theta_mech;

volatile int16_t    g_s2_foc_theta_zs;                                  /* delay for SRL(theta) */

volatile float32_t  g_f4_foc_theta_zi;                                  /* integral delay for PID(theta) */
volatile float32_t  g_f4_foc_theta_zd;                                  /* differential delay for PID(theta) */

volatile MTR_PID_PRM pid_foc_theta;                                     /* struct for PID(theta) Parameters */

volatile int16_t    g_s2_foc_omega;                                     /* speed [rad/s] : scale(Q4) */
volatile int16_t    g_s2_foc_omega_mech;

volatile int16_t    g_s2_foc_omega_zs;                                  /* delay for SRL(omega) */

volatile float32_t  g_f4_foc_omega_zi;                                  /* integral delay for PID(omega) */
volatile float32_t  g_f4_foc_omega_zd;                                  /* differential delay for PID(omega) */

volatile MTR_PID_PRM pid_foc_omega;                                     /* struct for PID(omega) Parameters */

volatile int16_t    g_s2_foc_id;                                        /* rotating coordinate id : scale(Q12) */

volatile float32_t  g_f4_foc_id_zi;                                     /* integral delay for PID(id) */
volatile float32_t  g_f4_foc_id_zd;                                     /* differential delay for PID(id) */

volatile MTR_PID_PRM pid_foc_id;                                        /* struct for PID(id) Parameters */

volatile int16_t    g_s2_foc_iq;                                        /* rotating coordinate iq : scale(Q12) */

volatile float32_t  g_f4_foc_iq_zi;                                     /* integral delay for PID(iq) */
volatile float32_t  g_f4_foc_iq_zd;                                     /* differential delay for PID(iq) */

volatile MTR_PID_PRM pid_foc_iq;                                        /* struct for PID(iq) Parameters */

volatile int16_t    g_s2_foc_ia;                                        /* stationary coordinate ia : scale(Q12) */
volatile int16_t    g_s2_foc_ib;                                        /* stationary coordinate ib : scale(Q12) */

volatile int16_t    g_s2_foc_ui;                                        /* U phase current : scale(Q12) */
volatile int16_t    g_s2_foc_vi;                                        /* V phase current : scale(Q12) */
volatile int16_t    g_s2_foc_wi;                                        /* W phase current : scale(Q12) */

volatile float32_t  g_f4_foc_coef_k;                                    /* coefficient(ratio) of converting current to voltage */

volatile int16_t    g_s2_foc_uv;                                        /* U phase voltage(PWM duty) [%] : scale(Q0) */
volatile int16_t    g_s2_foc_vv;                                        /* V phase voltage(PWM duty) [%] : scale(Q0) */
volatile int16_t    g_s2_foc_wv;                                        /* W phase voltage(PWM duty) [%] : scale(Q0) */

volatile uint16_t   g_u2_foc_ud;                                        /* U phase PWM duty : scale(Q0) */
volatile uint16_t   g_u2_foc_vd;                                        /* V phase PWM duty : scale(Q0) */
volatile uint16_t   g_u2_foc_wd;                                        /* W phase PWM duty : scale(Q0) */

volatile int16_t    g_s2_foc_uad_offset;                                /* U phase voltage of shunt resistor(offset) : scale(Q0) */
volatile int16_t    g_s2_foc_wad_offset;                                /* W phase voltage of shunt resistor(offset) : scale(Q0) */

volatile int16_t    g_s2_foc_uad;                                       /* U phase voltage of shunt resistor : scale(Q0) */
volatile int16_t    g_s2_foc_wad;                                       /* W phase voltage of shunt resistor : scale(Q0) */

volatile int16_t    g_s2_foc_adc_toggle;                                /* toggle switch for A/D conversion */

volatile int16_t    g_s2_foc_uadd;                                      /* uad' : scale(Q12) */
volatile int16_t    g_s2_foc_wadd;                                      /* wad' : scale(Q12) */

volatile int16_t    g_s2_foc_iad;                                       /* ia' : scale(Q12) */
volatile int16_t    g_s2_foc_ibd;                                       /* ib' : scale(Q12) */

volatile int16_t    g_s2_foc_iadd;                                      /* ia'' : scale(Q12) */
volatile int16_t    g_s2_foc_ibdd;                                      /* ib'' : scale(Q12) */

volatile int16_t    g_s2_foc_idd;                                       /* id' : scale(Q12) */
volatile int16_t    g_s2_foc_iqd;                                       /* iq' : scale(Q12) */

volatile uint16_t   g_u2_foc_flg_ctrl_duty;                             /* flag to control PWM duty */

/******************************************************************************
* Private global variables and functions
******************************************************************************/
volatile uint16_t   g_u2_cnt_timeout;                                   /* counter for timeout error */
volatile uint8_t    g_u1_direction;                                     /* rotational direction (0:CW ,1:CCW) */

volatile MTR_ICS_INPUT ics_input_buff;                                  /* struct for ICS input */

static void mtr_start_init(void);

static void mtr_start_init_extension(void);

/* for sequence control */
static uint8_t mtr_act_run(uint8_t);
static uint8_t mtr_act_stop(uint8_t);
static uint8_t mtr_act_reset(uint8_t);
static uint8_t mtr_act_error(uint8_t);
static uint8_t mtr_act_none(uint8_t);

static const volatile uint8_t g_u1_def_state[MTR_SIZE_EVENT][MTR_SIZE_STATE] = {
/*    state              0:MTR_MODE_STOP, 1:MTR_MODE_RUN,  2:MTR_MODE_ERROR */
/* event */
/* 0:MTR_EVENT_STOP  */{ MTR_MODE_STOP,   MTR_MODE_STOP,   MTR_MODE_ERROR },
/* 1:MTR_EVENT_RUN   */{ MTR_MODE_RUN,    MTR_MODE_RUN,    MTR_MODE_ERROR },
/* 2:MTR_EVENT_ERROR */{ MTR_MODE_ERROR,  MTR_MODE_ERROR,  MTR_MODE_ERROR },
/* 3:MTR_EVENT_RESET */{ MTR_MODE_STOP,   MTR_MODE_ERROR,  MTR_MODE_STOP  },};

static uint8_t (* gp_u1_def_action[MTR_SIZE_EVENT][MTR_SIZE_STATE])(uint8_t u1_state) = {
/*    state              0:MTR_MODE_STOP, 1:MTR_MODE_RUN,  2:MTR_MODE_ERROR */
/* event */
/* 0:MTR_EVENT_STOP  */{ mtr_act_none,    mtr_act_stop,    mtr_act_none  },
/* 1:MTR_EVENT_RUN   */{ mtr_act_run,     mtr_act_none,    mtr_act_none  },
/* 2:MTR_EVENT_ERROR */{ mtr_act_error,   mtr_act_error,   mtr_act_none  },
/* 3:MTR_EVENT_RESET */{ mtr_act_reset,   mtr_act_error,   mtr_act_reset },};

/******************************************************************************
* Function Name : R_MTR_InitSequence
* Description   : Initialize variables for sequence control
* Arguments     : none
* Return Value  : none
******************************************************************************/
void R_MTR_InitSequence(void)
{
    g_u1_flg_wait_stop = MTR_FLG_CLR;
    g_u2_cnt_wait_stop = 0;

    g_u1_mode_system = MTR_MODE_STOP;                                   /* system mode update : stop mode */
}

/******************************************************************************
* Function Name : R_MTR_ExecEvent
* Description   : Change motor state and select action
* Arguments     : u1_event - event
* Return Value  : none
******************************************************************************/
void R_MTR_ExecEvent(uint8_t u1_event)
{
    uint8_t u1_mode_system;

    u1_mode_system = g_u1_mode_system;

#ifdef FW_ERR_DET_UNKNOWN
    if (u1_event >= MTR_SIZE_EVENT)                                     /* check unknown event */
    {
        g_u1_mode_system = MTR_MODE_ERROR;                              /* system mode update : error mode */

        DI();

        g_u1_error_status |= MTR_UNKNOWN_ERROR;                         /* error status update : unknown error */

        EI();
    }
    else if (u1_mode_system >= MTR_SIZE_STATE)                          /* check unknown state */
    {
        g_u1_mode_system = MTR_MODE_ERROR;                              /* system mode update : error mode */

        DI();

        g_u1_error_status |= MTR_UNKNOWN_ERROR;                         /* error status update : unknown error */

        EI();
    }
    else
    {
        /* change motor state and select action */
        g_u1_mode_system = (gp_u1_def_action[u1_event][u1_mode_system])(g_u1_def_state[u1_event][u1_mode_system]);
    }
#else
    /* change motor state and select action */
    g_u1_mode_system = (gp_u1_def_action[u1_event][u1_mode_system])(g_u1_def_state[u1_event][u1_mode_system]);
#endif
}

/******************************************************************************
* Function Name : mtr_act_run
* Description   : Processing for motor control start
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_run(uint8_t u1_state)
{
    while (g_u1_flg_wait_stop == MTR_FLG_SET);                          /* waiting for motor stop */

    mtr_start_init();                                                   /* initialize variables when motor control start */

    mtr_ctrl_start();                                                   /* motor control start */

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_stop
* Description   : Processing for motor control stop
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_stop(uint8_t u1_state)
{
    mtr_ctrl_stop();                                                    /* motor control stop */

    g_u2_cnt_wait_stop = 0;
    g_u1_flg_wait_stop = MTR_FLG_SET;

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_none
* Description   : Do Nothing
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_none(uint8_t u1_state)
{
    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_reset
* Description   : Processing for motor control reset
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_reset(uint8_t u1_state)
{
    g_u1_flg_wait_stop = MTR_FLG_CLR;

    g_u1_enable_write = 0;

    g_s2_vdc_ad = 0;

#ifdef ICS_USE
/****** for ICS ******/
    g_u1_cnt_ics = 0;
/*********************/
#endif

    g_u1_error_status = 0;
    g_u1_mode_system = MTR_MODE_STOP;                                   /* set MTR_MODE_STOP */

    g_u2_cnt_timeout = 0;
    g_u1_direction = MTR_CW;                                            /* set MTR_CW */

    ics_input_buff.s2_theta = 0;
    ics_input_buff.s2_omega = 0;

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_error
* Description   : Processing when detecting an error
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_error(uint8_t u1_state)
{
    TRDOER1 = 0xFF;                                                     /* TRD output disable */

    mtr_ctrl_stop();                                                    /* motor control stop */

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_pattern_set
* Description   : Set voltage pattern
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_pattern_set(void)
{
    g_u2_cnt_timeout = 0;                                               /* clear counter for timeout error */

    if (g_u1_mode_system != MTR_MODE_ERROR)                             /* check system mode */
    {
        mtr_change_pattern();                                           /* change voltage pattern */
    }
}

/******************************************************************************
* Function Name : mtr_start_init
* Description   : Initialize variables when motor control start
* Arguments     : none
* Return Value  : none
******************************************************************************/
static void mtr_start_init(void)
{
    int16_t s2_temp;

    DI();

    s2_temp = mtr_get_adc(MTR_ADCCH_VM);

    EI();

    g_s2_vdc_ad = R_FIX8_mul_int16(s2_temp, MTR_VDC_SCALING);

    g_u2_cnt_timeout = 0;
}

/******************************************************************************
* Function Name : mtr_set_variables
* Description   : Set variables (conversion of unit)
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_set_variables(void)
{
    switch (g_s2_foc_encd_status)
    {
        case MTR_ENCD_STATE_POSITION:                                   /* position control */
            g_s2_foc_ref_theta = ics_input_buff.s2_theta;
        break;

        case MTR_ENCD_STATE_SPEED:                                      /* speed control */
            g_s2_foc_ref_omega = ics_input_buff.s2_omega;

            if (g_s2_foc_ref_omega > 0)                                 /* rotation direction */
            {
                g_u1_direction = MTR_CW;                                /* set MTR_CW */
            }
            else if (g_s2_foc_ref_omega < 0)
            {
                g_u1_direction = MTR_CCW;                               /* set MTR_CCW */
            }
            else
            {
                /* Do Nothing */
            }
        break;

        default: 
        break;
    }
}

/******************************************************************************
* Function Name : R_MTR_IcsInput
* Description   : Set value to buffer
* Arguments     : ICS input structure
* Return Value  : none
******************************************************************************/
void R_MTR_IcsInput(MTR_ICS_INPUT *ics_input)
{
    switch (g_s2_foc_encd_status)
    {
        case MTR_ENCD_STATE_POSITION:                                   /* position control */
            ics_input_buff.s2_theta = ics_input->s2_theta;
        break;

        case MTR_ENCD_STATE_SPEED:                                      /* speed control */
            ics_input_buff.s2_omega = ics_input->s2_omega;
        break;

        default: 
        break;
    }

    g_u1_enable_write = MTR_FLG_SET;
}

/******************************************************************************
* Function Name : R_MTR_SetSpeed
* Description   : Set speed reference
* Arguments     : ref_speed - Speed reference[rpm]
* Return Value  : none
******************************************************************************/
void R_MTR_SetSpeed(int16_t ref_speed)
{
    g_s2_foc_ref_omega = (int16_t)((float32_t)ref_speed * MTR_OMEGA_CALC_COEF);
}

/******************************************************************************
* Function Name : R_MTR_SetDir
* Description   : Set rotational direction
* Arguments     : dir - Rotational direction
* Return Value  : none
******************************************************************************/
void R_MTR_SetDir(uint8_t dir)
{
    g_u1_direction = dir;
}

/******************************************************************************
* Function Name : R_MTR_GetDir
* Description   : Get rotation direction
* Arguments     : none
* Return Value  : g_u1_direction - Rotational direction
******************************************************************************/
uint8_t R_MTR_GetDir(void)
{
    return (g_u1_direction);
}

/******************************************************************************
* Function Name : R_MTR_GetStatus
* Description   : Get status of motor control system
* Arguments     : none
* Return Value  : g_u1_mode_system - Motor status
******************************************************************************/
uint8_t R_MTR_GetStatus(void)
{
    return (g_u1_mode_system);
}

/******************************************************************************
* Function Name : mtr_error_check
* Description   : Check error status
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_error_check(void)
{
    /*==================================*/
    /*     over voltage error check     */
    /*==================================*/

#ifdef FW_ERR_DET_OVERVOLTAGE
    if (g_s2_vdc_ad > MTR_OVERVOLTAGE_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_OVER_VOLTAGE_ERROR;                    /* error status update : over voltage error */

        EI();
    }
#endif

    /*===================================*/
    /*     under voltage error check     */
    /*===================================*/

#ifdef FW_ERR_DET_UNDERVOLTAGE
    if (g_s2_vdc_ad < MTR_UNDERVOLTAGE_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_UNDER_VOLTAGE_ERROR;                   /* under voltage error */

        EI();
    }
#endif

    /*================================*/
    /*     over speed error check     */
    /*================================*/

#ifdef FW_ERR_DET_OVERSPEED
    if (abs(g_s2_foc_omega_mech) > MTR_SPEED_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_OVER_SPEED_ERROR;                      /* error status update : over speed error */

        EI();
    }
#endif

    /*===============================================================*/
    /*     timeout error check (undetected changing pwm pattern)     */
    /*===============================================================*/

#ifdef FW_ERR_DET_TIMEOUT
    if (g_u1_mode_system == MTR_MODE_RUN)                               /* check system mode */
    {
        g_u2_cnt_timeout++;
    }
    else
    {
        g_u2_cnt_timeout = 0;
    }

    if (g_u2_cnt_timeout > MTR_TIMEOUT_CNT)                             /* check undetected time */
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_TIMEOUT_ERROR;                         /* error status update : timeout error */

        EI();
    }
#endif
}

/******************************************************************************
* Function Name : R_MTR_InitEncoderVector
* Description   : Initialize variables for encoder vector control
* Arguments     : none
* Return Value  : none
******************************************************************************/
void R_MTR_InitEncoderVector(void)
{
    g_s2_foc_encd_status = MTR_ENCD_STATE_ALIGN;

    g_s2_foc_mech_count = 0;
    g_s2_foc_elec_count = 0;

    g_s2_foc_ref_theta = MTR_FIXED_THETA;
    g_s2_foc_ref_omega = MTR_FIXED_OMEGA;

    g_s2_foc_theta = 0;
    g_s2_foc_theta_elec = 0;
    g_s2_foc_theta_mech = 0;

    g_s2_foc_theta_zs = 0;

    g_f4_foc_theta_zi = 0.0f;
    g_f4_foc_theta_zd = 0.0f;

    pid_foc_theta.f4_kp = MTR_PID_THETA_KP;
    pid_foc_theta.f4_ti = MTR_PID_THETA_TI;
    pid_foc_theta.f4_ki = MTR_PID_THETA_KI;
    pid_foc_theta.f4_td = MTR_PID_THETA_TD;
    pid_foc_theta.f4_kd = MTR_PID_THETA_KD;
    pid_foc_theta.f4_kt = MTR_PID_THETA_KT;

    pid_foc_theta.f4_zi = &g_f4_foc_theta_zi;
    pid_foc_theta.f4_zd = &g_f4_foc_theta_zd;

    g_s2_foc_omega = 0;
    g_s2_foc_omega_mech = 0;

    g_s2_foc_omega_zs = 0;

    g_f4_foc_omega_zi = 0.0f;
    g_f4_foc_omega_zd = 0.0f;

    pid_foc_omega.f4_kp = MTR_PID_OMEGA_KP;
    pid_foc_omega.f4_ti = MTR_PID_OMEGA_TI;
    pid_foc_omega.f4_ki = MTR_PID_OMEGA_KI;
    pid_foc_omega.f4_td = MTR_PID_OMEGA_TD;
    pid_foc_omega.f4_kd = MTR_PID_OMEGA_KD;
    pid_foc_omega.f4_kt = MTR_PID_OMEGA_KT;

    pid_foc_omega.f4_zi = &g_f4_foc_omega_zi;
    pid_foc_omega.f4_zd = &g_f4_foc_omega_zd;

    g_s2_foc_id = 0;

    g_f4_foc_id_zi = 0.0f;
    g_f4_foc_id_zd = 0.0f;

    pid_foc_id.f4_kp = MTR_PID_ID_KP;
    pid_foc_id.f4_ti = MTR_PID_ID_TI;
    pid_foc_id.f4_ki = MTR_PID_ID_KI;
    pid_foc_id.f4_td = MTR_PID_ID_TD;
    pid_foc_id.f4_kd = MTR_PID_ID_KD;
    pid_foc_id.f4_kt = MTR_PID_ID_KT;

    pid_foc_id.f4_zi = &g_f4_foc_id_zi;
    pid_foc_id.f4_zd = &g_f4_foc_id_zd;

    g_s2_foc_iq = 0;

    g_f4_foc_iq_zi = 0.0f;
    g_f4_foc_iq_zd = 0.0f;

    pid_foc_iq.f4_kp = MTR_PID_IQ_KP;
    pid_foc_iq.f4_ti = MTR_PID_IQ_TI;
    pid_foc_iq.f4_ki = MTR_PID_IQ_KI;
    pid_foc_iq.f4_td = MTR_PID_IQ_TD;
    pid_foc_iq.f4_kd = MTR_PID_IQ_KD;
    pid_foc_iq.f4_kt = MTR_PID_IQ_KT;

    pid_foc_iq.f4_zi = &g_f4_foc_iq_zi;
    pid_foc_iq.f4_zd = &g_f4_foc_iq_zd;

    g_s2_foc_ia = 0;
    g_s2_foc_ib = 0;

    g_s2_foc_ui = 0;
    g_s2_foc_vi = 0;
    g_s2_foc_wi = 0;

    DI();

    g_s2_pdrv_ad = mtr_get_adc(MTR_ADCCH_PDRV);

    EI();

    g_f4_foc_coef_k = MTR_COEF_K / (float32_t)g_s2_pdrv_ad;

    g_s2_foc_uv = 0;
    g_s2_foc_vv = 0;
    g_s2_foc_wv = 0;

    g_u2_foc_ud = (uint16_t)MTR_START_CARRIER_SET;
    g_u2_foc_vd = (uint16_t)MTR_START_CARRIER_SET;
    g_u2_foc_wd = (uint16_t)MTR_START_CARRIER_SET;

    {   /* get offset of A/D conversion */
        int16_t s2_temp_iu, s2_temp_iw, s2_cnt;

        s2_temp_iu = 0;
        s2_temp_iw = 0;

        DI();

        for (s2_cnt = 0; s2_cnt < 8; s2_cnt++)
        {
            s2_temp_iu += mtr_get_adc(MTR_ADCCH_IU);
            s2_temp_iw += mtr_get_adc(MTR_ADCCH_IW);
        }

        EI();

        g_s2_foc_uad_offset = MTR_CENTER_ADC - (s2_temp_iu >> 3);
        g_s2_foc_wad_offset = MTR_CENTER_ADC - (s2_temp_iw >> 3);
    }

    g_s2_foc_uad = 0;
    g_s2_foc_wad = 0;

    g_s2_foc_adc_toggle = MTR_ADCCH_IU;

    g_s2_foc_uadd = 0;
    g_s2_foc_wadd = 0;

    g_s2_foc_iad = 0;
    g_s2_foc_ibd = 0;

    g_s2_foc_iadd = 0;
    g_s2_foc_ibdd = 0;

    g_s2_foc_idd = 0;
    g_s2_foc_iqd = 0;

    mtr_calc_InitVariables();                                           /* initialize private global variables */

    g_u2_foc_flg_ctrl_duty = MTR_FLG_CLR;
}

/******************************************************************************
* Function Name : R_MTR_SetFlagCtrlDuty
* Description   : Set/Clear a flag to control PWM duty
* Arguments     : u2_flag - set/clear
* Return Value  : none
******************************************************************************/
void R_MTR_SetFlagCtrlDuty(uint16_t u2_flag)
{
    g_u2_foc_flg_ctrl_duty = u2_flag;
}

